@@ -60,7 +60,6 @@ SOFTWARE.
 #include <X11/extensions/XInput.h>
 #include <X11/extensions/extutil.h>
 #include "XIint.h"
-#include <limits.h>
 
 /* Calculate length field to a multiples of sizeof(XID). XIDs are typedefs
  * to ulong and thus may be 8 bytes on some platforms. This can trigger a
@@ -73,7 +72,7 @@ static int pad_to_xid(int base_size)
     return ((base_size + padsize - 1)/padsize) * padsize;
 }
 
-static size_t
+static int
 SizeClassInfo(xAnyClassPtr *any, int num_classes)
 {
     int size = 0;
@@ -107,7 +106,7 @@ SizeClassInfo(xAnyClassPtr *any, int num_classes)
 static void
 ParseClassInfo(xAnyClassPtr *any, XAnyClassPtr *Any, int num_classes)
 {
-    int j, k;
+    int j;
 
     for (j = 0; j < num_classes; j++) {
         switch ((*any)->class) {
@@ -135,6 +134,7 @@ ParseClassInfo(xAnyClassPtr *any, XAnyClassPtr *Any, int num_classes)
                 }
             case ValuatorClass:
                 {
+                    int k;
                     XValuatorInfoPtr V = (XValuatorInfoPtr) *Any;
                     xValuatorInfoPtr v = (xValuatorInfoPtr) *any;
                     XAxisInfoPtr A;
@@ -169,7 +169,7 @@ XListInputDevices(
     register Display	*dpy,
     int			*ndevices)
 {
-    size_t size;
+    int size;
     xListInputDevicesReq *req;
     xListInputDevicesReply rep;
     xDeviceInfo *list, *slist = NULL;
@@ -177,9 +177,9 @@ XListInputDevices(
     XDeviceInfo *clist = NULL;
     xAnyClassPtr any, sav_any;
     XAnyClassPtr Any;
-    unsigned char *nptr, *Nptr;
+    char *nptr, *Nptr;
     int i;
-    unsigned long rlen;
+    long rlen;
     XExtDisplayInfo *info = XInput_find_display(dpy);
 
     LockDisplay(dpy);
@@ -198,12 +198,11 @@ XListInputDevices(
 
     if ((*ndevices = rep.ndevices)) {	/* at least 1 input device */
 	size = *ndevices * sizeof(XDeviceInfo);
-	if (rep.length < (INT_MAX >> 2)) {
-	    rlen = rep.length << 2;	/* multiply length by 4    */
-	    slist = list = Xmalloc(rlen);
-	}
+	rlen = rep.length << 2;	/* multiply length by 4    */
+	list = (xDeviceInfo *) Xmalloc(rlen);
+	slist = list;
 	if (!slist) {
-	    _XEatDataWords(dpy, rep.length);
+	    _XEatData(dpy, (unsigned long)rlen);
 	    UnlockDisplay(dpy);
 	    SyncHandle();
 	    return (XDeviceInfo *) NULL;
@@ -216,12 +215,9 @@ XListInputDevices(
             size += SizeClassInfo(&any, (int)list->num_classes);
 	}
 
-	Nptr = ((unsigned char *)list) + rlen + 1;
-	for (i = 0, nptr = (unsigned char *)any; i < *ndevices; i++) {
+	for (i = 0, nptr = (char *)any; i < *ndevices; i++) {
 	    size += *nptr + 1;
 	    nptr += (*nptr + 1);
-	    if (nptr > Nptr)
-		goto out;
 	}
 
 	clist = (XDeviceInfoPtr) Xmalloc(size);
@@ -247,8 +243,8 @@ XListInputDevices(
 	}
 
 	clist = sclist;
-	nptr = (unsigned char *)any;
-	Nptr = (unsigned char *)Any;
+	nptr = (char *)any;
+	Nptr = (char *)Any;
 	for (i = 0; i < *ndevices; i++, clist++) {
 	    clist->name = (char *)Nptr;
 	    memcpy(Nptr, nptr + 1, *nptr);
@@ -258,7 +254,6 @@ XListInputDevices(
 	}
     }
 
-  out:
     XFree((char *)slist);
     UnlockDisplay(dpy);
     SyncHandle();
